######################################################################
#
# Script to process Arctic Composite Imagery - Shortwave Infrared Version
#
# Changed log file locations
#
#
#
#
######################################################################

#############################
# *** UTILITY FUNCTIONS *** #
#############################


# Passed the file of times in $1 and the time of our new image in $2
# A bit nasty due to bash not wanting to nicely pass "exists" back through.
check_times()
{
  # Checks to within 5 minutes.
  btime=`expr $2 - 500`
  etime=`expr $2 + 500`
  exists=0
  retcode=$(
    cat $1 | {
      while read timeslice; do
        if [ "$timeslice" -lt "$btime" ] || [ "$timeslice" -gt "$etime" ]; then
          continue
        else
          exists=1
        fi
      done
      echo $exists
    }
  )

  if [ $retcode -eq 0 ]; then
    echo "retcode: $retcode, $2 >> $1"
    echo $2 >> $1
  else
    echo "retcode: $retcode!, not adding!"
  fi

  return $retcode
}


# A replacement for the "wait" command, since we don't
# run everything in subshells.
watch_me()
{ 
  echo "*** Watch for $1 started at `date +%H:%M:%S`. ***";
  while ps -ef | grep $1 | grep -v grep > /dev/null
  do
    sleep 5
  done;
  echo "*** Watch for $1 completed at `date +%H:%M:%S`. ***"; 
}



#######################
# *** MAIN SCRIPT *** #
#######################

#
# Initialize directories/etc.
#
. $HOME/.compositerc-sw

#
# Load our base config file for processing.
#
. $HOME/.processrc-sw


#
# Now overwrite any defaults with command-line arguments we may have gotten
#
while getopts ":d:w:l:r:p:P:t:s:e:" opt; do
  case $opt in
    d ) ROOT_DATADIR=$OPTARG;;
    w ) PDELAY=$OPTARG;;
    l ) COMPLAT=$OPTARG;;
    r ) REMAP_DIR=$OPTARG;;
    p ) POLE=$OPTARG;;
    t ) COMPTIME=$OPTARG;;
    s ) TSTART=$OPTARG;;
    e ) TEND=$OPTARG;;
    \? ) echo 'bad argument'
         exit 1
    esac
done

#
# Reset data directory and logfile directory
#
ROOT_DATADIR=$DATAN
LOG_DATADIR=$HOME/logs/logs-sw

#
# Set up and redirect the logfiles.
#
LOGFILE=${LOG_DATADIR}/np-sw-compfiles.log
rm $LOGFILE
exec > $LOGFILE
exec 2> $LOGFILE


#
# Set up all our direcotires
#

mkdir -p $REMAP_DIR

DEBUG_LOG_DIR=$LOG_DATADIR/logs
mkdir -p $DEBUG_LOG_DIR


COMP_LOG=${LOG_DATADIR}/compositefiles-sw.log
SMALL_LOG=${LOG_DATADIR}/compositesmall-sw.log


MCPATH=$REMAP_DIR:$MCIDAS_DATA
export MCPATH REMAP_DIR PATH COMPTIME DEBUG_LOG_DIR LOGFILE

echo `date`

echo " ********************************  print out variables  ********************************"
echo "COMPTIME= ${COMPTIME}"
echo "ROOT_DATADIR= ${ROOT_DATADIR}"
echo "LOG_DATADIR= ${LOG_DATADIR}"
echo "PDELAY= ${PDELAY}"
echo "COMPLAT= ${COMPLAT}"
echo "REMAP_DIR= ${REMAP_DIR}"
echo "POLE= ${POLE}"
echo "TSTART= ${TSTART}"
echo "TEND= ${TEND}"
echo "PATH= ${PATH}"
echo "MCPATH= ${MCPATH}"
echo "DEBUG_LOG_DIR= ${MCPATH}"
echo " ********************************  print out variables  ********************************"


#
# Set up all the relevant times to our compositing.
#
imgyyddd=${COMPTIME:0:5}

imgChhmmss="${COMPTIME:5:2}:${COMPTIME:7:2}:${COMPTIME:9:2}"
imghhmmss="${COMPTIME:5:2}${COMPTIME:7:2}${COMPTIME:9:2}"
imghhmm="${COMPTIME:5:2}${COMPTIME:7:2}"

echo "COMPTIME $COMPTIME / TSTART $TSTART / TEND $TEND"
echo "imgyyddd $imgyyddd / imgChhmmss $imgChhmmss"

rm $REMAP_DIR/AREA0*
rm $REMAP_DIR/AREA1*
rm $REMAP_DIR/AREA2*
rm $REMAP_DIR/AREA3*
rm $REMAP_DIR/AREA4*
rm $REMAP_DIR/AREA5*
rm $REMAP_DIR/AREA6*
rm $REMAP_DIR/AREA7*
rm $REMAP_DIR/AREA8*
rm $REMAP_DIR/AREA9*
rm $REMAP_DIR/OVCHK*

#
#look for new data
#
cd $PROCESS_DIR
unset image_list

dsserve.k ADD GEO/COMP AREA 1 9999

#
# Create basemap for both south pole and north pole. 
#
cp $REQUIRED_FILES_DIR/initialized_area $REMAP_DIR/AREA5001

echo " "
echo " ********************************  Create Basemaps  **********************************"
echo "IMGREMAP GEO/COMP.5001 GEO/COMP.8002 PRO=${PROJECTION} LAT=${NP_COMPLAT} ${NP_COMPLON} RES=${RES_KM} SIZE=${SIZELINE} ${SIZEELE} "
imgremap.k GEO/COMP.5001 GEO/COMP.8002 PRO=${PROJECTION} LAT=${NP_COMPLAT} ${NP_COMPLON} RES=${RES_KM} SIZE=${SIZELINE} ${SIZEELE} 

echo "IMGREMAP GEO/COMP.5001 GEO/COMP.8004 PRO=${PROJECTION} LAT=${NP_COMPLAT} ${NP_COMPLON} RES=${REDUCED_RES_KM} SIZE=${REDUCED_SIZELINE} ${REDUCED_SIZEELE} "
imgremap.k GEO/COMP.5001 GEO/COMP.8004 PRO=${PROJECTION} LAT=${NP_COMPLAT} ${NP_COMPLON} RES=${REDUCED_RES_KM} SIZE=${REDUCED_SIZELINE} ${REDUCED_SIZEELE} 

echo " ********************************  Done with  Basemaps  **********************************"
echo " "

echo -e "\n************************\n" >> $COMP_LOG
echo "$imgChhmmss"  >> $COMP_LOG


imgnum=0

echo `date`
#
# PRIORITY_FILE contains a list of datasets that will be processed (East, West, POES ...
# The list is sorted in the order that we want to paste things, and includes
# some details about the img type (like whether it is a geopositional, etc.)
#

echo " "
echo "************************** begin  priority file  ***************************"
cat $PRIORITY_FILE
echo "************************** end priority file  ***************************"
echo " "

cat $PRIORITY_FILE|while read uniquestr geopos lowscale highscale ovchk; do

  echo "****** $uniquestr ******"

#
# Now go through the list of files that need to be processed (*.todo)
# Each file will be checked to satisfy several criteria
#
  ls -1|grep .*$uniquestr.*todo|while read image ; do

     sat=$(cat ${image}|awk -F" " '{print $1 }')
     day=$(cat ${image}|awk -F" " '{print $2 }')
     time=$(cat ${image}|awk -F" " '{print $3 }')
     band=$(cat ${image}|awk -F" " '{print $4 }')

     time=`printf %06d $time`  # Convert value to 6 digit integer - some between 0 and 10 aren't

     # If the image's time is within our composite's boundaries...
     if [ "${day}${time}" -ge "$TSTART" ] && [ "${day}${time}" -le "$TEND" ]; then

       # and if the image type is set as needing to check for multiple types (ie: WALLOPS and DDS)
       if [ $ovchk -gt 0 ]; then

         # ... check to make sure that a copy (ie: DDS) hasn't already been pasted on the composite...
         check_times "${REMAP_DIR}/OVCHK_O${ovchk}" $time
         if [ $? -eq 1 ]; then
           # ... if it has been, skip this image and go on to the next.
           continue
         fi
       fi

#
# Rename the file so it can be used in standard McIDAS commands
#
       echo "$image"
       cp ${image%.todo} $REMAP_DIR/AREA9001
       cd $REMAP_DIR
       
# Get number of last AREA7* file...the growing list of remapped files
       ls -1 AREA7*
       if [ $? -eq 0 ] ; then
         newestarea=`ls -1 AREA7* | tail -n 1`
         imgnum=${newestarea:5:3}
       else
         imgnum=0
       fi

#
# This piece of code protects from hexidecimal problems
#
       echo "f imgnum [$imgnum]"
       imgnum=$((10#$imgnum+1))
       echo "m imgnum [$imgnum]"

       imgparm_src=$((6500+$imgnum))
       imgparm_dest=$((6000+$imgnum))
       np_remap=$((7000+$imgnum))

       echo "np_remap [$np_remap]"

#
#                          North Pole Processing
#
#     This command verifies if the geographical region of the incoming
#     image is within the geographical region of our basemap.  To make 
#     it a little quicker, we reduced the resolution of the basemap
#
       echo " "
       echo "Checking to see if image matches geographical bounds"
       echo "IMGREMAP GEO/COMP.9001 GEO/COMP.8004"
       imgremap.k GEO/COMP.9001 GEO/COMP.8004
       echo " "

       if [ $? -eq 0 ]; then
         
         cp $REMAP_DIR/AREA8002 AREA${np_remap}
         cp $REMAP_DIR/AREA9001 AREA${imgparm_src}
         imglist.k GEO/COMP.${imgparm_src} DEV=T imgparm.log A
#
# Start of subshell
# ( opens a subshell that goes to another core to send off the imgparms and remap2s all off essentially at once )
#
         ( echo "imgparm.k GEO/COMP.${imgparm_src} GEO/COMP.${imgparm_dest}  BASE=$imgyyddd $imgChhmmss";
         imgparm.k GEO/COMP.${imgparm_src} GEO/COMP.${imgparm_dest}  BASE=$imgyyddd $imgChhmmss;

#
# REMAP the data
#
         echo "remap2.k ${imgparm_dest} ${np_remap} SMO=YES BAND=1 2 3 4 5 6 7 8";
         remap2.k ${imgparm_dest} ${np_remap} SMO=YES BAND=1 2 3 4 5 6 7 8; ) &

# End of subshell
       else
         rm AREA9001
       fi
        
       cd $PROCESS_DIR

       echo "$uniquestr $image" >> $COMP_LOG
     fi
  done
done

# Functions defined above - these are important..
# A poor mans "wait" for all of the subshells...
# These next two lines might have to be rewritten to use wait command...

watch_me imgparm.k
watch_me remap2.k


cd $REMAP_DIR
newestarea=`ls -1 AREA7* | tail -n 1`
imgnum=${newestarea:5:3}
pwd        
echo $MCPATH
# echo "dmap.k AREA3"
#dmak.k AREA3

# Fix this - increase the number of file above 100

dsserve.k ADD NP/LIST AREA 7001 7100
dsserve.k ADD NP/FINAL AREA 7101 7101

echo "satcomp.k NP/LIST NP/LIST.100 1 ${imgnum} BAND=1 TOLER=15 3600"
satcomp.k NP/LIST NP/LIST.100 1 ${imgnum} BAND=1 TOLER=15 3600

#
# Fix the image times on the composites
#

echo" imgcha.k NP/LIST.100 DAY=$imgyyddd TIME=$imgChhmmss SS=401 MEMO='Arctic Shortwave Composite' OTYP=VISR BAND=2"
imgcha.k NP/LIST.100 DAY=$imgyyddd TIME=$imgChhmmss SS=401 MEMO='Arctic Shortwave Composite' OTYP=VISR BAND=2

echo " "
echo "lwu.k POKE AREA7100 $imghhmmss 46"
lwu.k POKE AREA7100 $imghhmmss 46

echo " "
echo "imgcopy.k NP/LIST.100 NP/FINAL.1 SIZE=SAME STYPE=VISR"
imgcopy.k NP/LIST.100 NP/FINAL.1 SIZE=SAME STYPE=VISR

#
#  If the imgcopy fails, send e-mail alert
#
if [ $? -eq 1 ]; then
 echo "Error: No Arctic Shortwave Infrared Satellite Composite" | mail -s "Arctic Shortwave Infrared Satellite Composite" $EMAIL
fi

#
# If the satellite composite crosses a standard deviation threshold, send e-mail alert
#
stndev=`stndev.k 7101 | awk '{print $4}'`
echo "The Standard Deviation is" $stndev
if [ $stndev -lt 1000 ]; then
 echo "Warning: Arctic Shortwave Infrared Satellite Composite out of specifications: $stndev" | mail -s "Warning: Arctic Shortwave Infrared Satellite Composite" $EMAIL
fi

#
# Commenting out - not checking high standard deviation
#
#if [ $stndev -gt 4000 ]; then
# echo "Warning: Arctic Shortwave Infrared Satellite Composite out of specifications: $stndev" | mail -s "Warning: Arctic Shortwave Infrared Satellite Composite" $EMAIL
#fi

echo " "
echo "cp $REMAP_DIR/AREA7101 ${ROOT_DATADIR}/arctic/archive/shortwave/areas/arctic-${COMPTIME}.area"
cp $REMAP_DIR/AREA7101 ${ROOT_DATADIR}/arctic/archive/shortwave/areas/arctic-${COMPTIME}.area

#
# Copy the composites to our viewable directory.
#
mkdir -p ${ROOT_DATADIR}/arctic
mkdir -p ${ROOT_DATADIR}/arctic/archive
mkdir -p ${ROOT_DATADIR}/arctic/archive/shortwave
# Could remove above lines...
mkdir -p ${ROOT_DATADIR}/arctic/archive/shortwave/areas
mkdir -p ${ROOT_DATADIR}/arctic/archive/shortwave/gif
# Could remove the above line..
mkdir -p ${ROOT_DATADIR}/arctic/archive/shortwave/gif/latest
mkdir -p ${ROOT_DATADIR}/arctic/archive/shortwave/gif/small
mkdir -p ${ROOT_DATADIR}/arctic/archive/shortwave/gif/medium
mkdir -p ${ROOT_DATADIR}/arctic/archive/shortwave/gif/large
mkdir -p ${ROOT_DATADIR}/arctic/archive/shortwave/gif/extra-large


#
# start up a mcenv session to display the images at different resolutions
#
mcenv -i 192 -g 32 -f 1@2188x2176 -f 1@1100x1088 -f 1@738x726 -f 1@556x544 << 'EOF'

#
# Create netCDF file and post on LDM
#
echo dsserve.k ADD NETCDF/COMPSW NCDF 1 1 TYPE=IMAGE \"netCDF COMPOSITE SW IMAGE
dsserve.k ADD NETCDF/COMPSW NCDF 1 1 TYPE=IMAGE \"netCDF COMPOSITE SW IMAGE
echo imgcopy.k NP/FINAL.1 NETCDF/COMPSW.1 SIZE=ALL
imgcopy.k NP/FINAL.1 NETCDF/COMPSW.1 SIZE=ALL
echo dmap.k *.nc
dmap.k *.nc

echo start AWIPS netCDF file
dataloc.k ADD ARCAWIP LOCAL-DATA
dsserve.k ADD ARCAWIP/SW AWIP TYPE=IMAGE DIR='$ROOT_DATADIR/arctic/archive/shortwave/netcdf/.' \"AWIPS II netCDF for Arctic SW Composite
imgcopy.k NP/FINAL.1 ARCAWIP/SW.7990 SIZE=ALL
echo end AWIPS netCDF file

#
# Display and Save Arctic composite at full res, 1/2 res, 1/3 res and 1/4 res
#
imgdisp.k NP/FINAL.1 1 
sf.k 1
frmlabel.k JUST=C \"ARCTIC COMPOSITE SHORTWAVE IMAGE \(DAY\) AT \(HH\) UTC AMRC\:\(LOGO\) 
map.k X 1 LALO 2
gu.k MAKE 1 RED
frmsave.k 1 ${ROOT_DATADIR}/arctic/archive/shortwave/gif/extra-large/arctic-${COMPTIME}.gif
frmsave.k 1 ${ROOT_DATADIR}/arctic/archive/shortwave/gif/latest/latest-extra-large-arctic.gif

imgdisp.k NP/FINAL.1 2 MAG=-2 
sf.k 2
frmlabel.k JUST=C \"ARCTIC COMPOSITE SHORTWAVE IR IMAGE \(DAY\) AT \(HH\) UTC AMRC\:\(LOGO\) 
map.k X 1 LALO 2
gu.k MAKE 1 RED
frmsave.k 2 ${ROOT_DATADIR}/arctic/archive/shortwave/gif/large/arctic-${COMPTIME}.gif
frmsave.k 2 ${ROOT_DATADIR}/arctic/archive/shortwave/gif/latest/latest-large-arctic.gif

imgdisp.k NP/FINAL.1 3 MAG=-3 
sf.k 3
frmlabel.k JUST=C \"ARCTIC COMPOSITE SHORTWAVE IR \(DAY\) AT \(HH\) UTC AMRC\:\(LOGO\) 
map.k X 1 LALO 2
gu.k MAKE 1 RED
frmsave.k 3 ${ROOT_DATADIR}/arctic/archive/shortwave/gif/medium/arctic-${COMPTIME}.gif
frmsave.k 3 ${ROOT_DATADIR}/arctic/archive/shortwave/gif/latest/latest-medium-arctic.gif

imgdisp.k NP/FINAL.1 4 MAG=-4 
sf.k 4
frmlabel.k JUST=C \"ARCTIC SW IR \(DAY\) AT \(HH\) UTC AMRC\:\(LOGO\) 
map.k X 1 LALO 2
gu.k MAKE 1 RED
frmsave.k 4 ${ROOT_DATADIR}/arctic/archive/shortwave/gif/small/arctic-${COMPTIME}.gif
frmsave.k 4 ${ROOT_DATADIR}/arctic/archive/shortwave/gif/latest/latest-small-arctic.gif
echo after frmsave

# Adding Code to gzip the area file before posting to LDM
# Changing the name on the LDM product to match CIMSS LDM expectations for
#  distribution to GINA (NWS AK) and OPC
# M. Lazzara Nov 6, 2012

ldmyyyymmdd=`imglist.k NP/FINAL.1 | grep COMP | awk '{print $6}' | cut -c1-5`
echo ${ldmyyyymmdd}
ldmdate=`/home/mcidas/bin/datelist.k ${ldmyyyymmdd} FORM='(YYYY)(MM)(DD)' | awk '{print $1}'`
echo ${ldmdate}
ldmhh=`imglist.k NP/FINAL.1 | grep COMP |awk '{print $7}' |cut -c1-2`
ldmmm=`imglist.k NP/FINAL.1 | grep COMP |awk '{print $7}' |cut -c4-5`
echo ${ldmhh}
echo ${ldmmm}

# netCDF

echo /home/ldm/bin/pqinsert -v -q /home/ldm/var/queues/ldm.pq -f EXP -p SSEC_WEB_AMRC-ARCTIC_4KM_SWCOMP_${ldmdate}_${ldmhh}${ldmmm}.nc $HOME/remap_dir-sw/COMPSW0001.nc
/home/ldm/bin/pqinsert -v -q /home/ldm/var/queues/ldm.pq -f EXP -p SSEC_WEB_AMRC-ARCTIC_4KM_SWCOMP_${ldmdate}_${ldmhh}${ldmmm}.nc $HOME/remap_dir-sw/COMPSW0001.nc

# McIDAS AREA

/home/ldm/bin/pqinsert -v -q /home/ldm/var/queues/ldm.pq -f EXP -p SSEC_WEB_AMRC-ARCTIC_4KM_SWCOMP_${ldmdate}_${ldmhh}${ldmmm}.area $REMAP_DIR/AREA7101

# GIF files

/home/ldm/bin/pqinsert -v -q /home/ldm/var/queues/ldm.pq -f EXP -p SSEC_WEB_AMRC-ARCTIC_4KM_SWCOMP_${ldmdate}_${ldmhh}${ldmmm}_xlarge.gif ${ROOT_DATADIR}/arctic/archive/shortwave/gif/latest/latest-extra-large-arctic.gif

/home/ldm/bin/pqinsert -v -q /home/ldm/var/queues/ldm.pq -f EXP -p SSEC_WEB_AMRC-ARCTIC_4KM_SWCOMP_${ldmdate}_${ldmhh}${ldmmm}_large.gif ${ROOT_DATADIR}/arctic/archive/shortwave/gif/latest/latest-large-arctic.gif

/home/ldm/bin/pqinsert -v -q /home/ldm/var/queues/ldm.pq -f EXP -p SSEC_WEB_AMRC-ARCTIC_4KM_SWCOMP_${ldmdate}_${ldmhh}${ldmmm}_medium.gif ${ROOT_DATADIR}/arctic/archive/shortwave/gif/latest/latest-medium-arctic.gif

/home/ldm/bin/pqinsert -v -q /home/ldm/var/queues/ldm.pq -f EXP -p SSEC_WEB_AMRC-ARCTIC_4KM_SWCOMP_${ldmdate}_${ldmhh}${ldmmm}_small.gif ${ROOT_DATADIR}/arctic/archive/shortwave/gif/latest/latest-small-arctic.gif


# AREA File

cp ${ROOT_DATADIR}/arctic/archive/shortwave/areas/arctic-${COMPTIME}.area ${ROOT_DATADIR}/arctic/archive/shortwave/areas/arctic-${COMPTIME}.area.copy
gzip ${ROOT_DATADIR}/arctic/archive/shortwave/areas/arctic-${COMPTIME}.area.copy
echo Made it below gzip
/home/ldm/bin/pqinsert -v -q /home/ldm/var/queues/ldm.pq -f EXP -p SSEC_NAWIPS_AMRC-ARCTIC_4KM_SWCOMP_${ldmdate}_${ldmhh}${ldmmm}.gz ${ROOT_DATADIR}/arctic/archive/shortwave/areas/arctic-${COMPTIME}.area.copy.gz
/bin/rm ${ROOT_DATADIR}/arctic/archive/shortwave/areas/arctic-${COMPTIME}.area.copy.gz


EOF


# CLEANUP!
#
# purge old lines from the composite list
#
tail -10000 $COMP_LOG > /tmp/.processdata.tmp
cat /tmp/.processdata.tmp > $COMP_LOG
rm -f /tmp/.processdata.tmp
tail -1000 $COMP_LOG > $SMALL_LOG

cp $LOGFILE $DEBUG_LOG_DIR/log.${COMPTIME}
